@@ -12,10 +12,11 @@ from ..exceptions import ParseError
 
 
 def is_flag(value):
-    return value.startswith('-')
+    return value.startswith("-")
+
 
 def is_long_flag(value):
-    return value.startswith('--')
+    return value.startswith("--")
 
 
 class Parser(object):
@@ -38,6 +39,7 @@ class Parser(object):
 
     .. versionadded:: 1.0
     """
+
     def __init__(self, contexts=(), initial=None, ignore_unknown=False):
         self.initial = initial
         self.contexts = Lexicon()
@@ -59,9 +61,10 @@ class Parser(object):
         """
         Parse an argv-style token list ``argv``.
 
-        Returns a list of ``Context`` objects matching the order they were
-        found in the ``argv`` and containing ``Argument`` objects with updated
-        values based on any flags given.
+        Returns a list (actually a subclass, `.ParseResult`) of
+        `.ParserContext` objects matching the order they were found in the
+        ``argv`` and containing `.Argument` objects with updated values based
+        on any flags given.
 
         Assumes any program name has already been stripped out. Good::
 
@@ -71,24 +74,32 @@ class Parser(object):
 
             Parser(...).parse_argv(['invoke', '--core-opt', ...])
 
+        :param argv: List of argument string tokens.
+        :returns:
+            A `.ParseResult` (a ``list`` subclass containing some number of
+            `.ParserContext` objects).
+
         .. versionadded:: 1.0
         """
-        machine = ParseMachine(initial=self.initial, contexts=self.contexts,
-            ignore_unknown=self.ignore_unknown)
+        machine = ParseMachine(
+            initial=self.initial,
+            contexts=self.contexts,
+            ignore_unknown=self.ignore_unknown,
+        )
         # FIXME: Why isn't there str.partition for lists? There must be a
         # better way to do this. Split argv around the double-dash remainder
         # sentinel.
-        debug("Starting argv: {!r}".format(argv,))
+        debug("Starting argv: {!r}".format(argv))
         try:
-            ddash = argv.index('--')
+            ddash = argv.index("--")
         except ValueError:
-            ddash = len(argv) # No remainder == body gets all
+            ddash = len(argv)  # No remainder == body gets all
         body = argv[:ddash]
-        remainder = argv[ddash:][1:] # [1:] to strip off remainder itself
+        remainder = argv[ddash:][1:]  # [1:] to strip off remainder itself
         if remainder:
-            debug("Remainder: argv[{!r}:][1:] => {!r}".format(
-                ddash, remainder
-            ))
+            debug(
+                "Remainder: argv[{!r}:][1:] => {!r}".format(ddash, remainder)
+            )
         for index, token in enumerate(body):
             # Handle non-space-delimited forms, if not currently expecting a
             # flag value and still in valid parsing territory (i.e. not in
@@ -98,11 +109,12 @@ class Parser(object):
             # previously seen flag optionally takes a value.
             mutations = []
             orig = token
-            if (is_flag(token) and not machine.result.unparsed):
+            if is_flag(token) and not machine.result.unparsed:
                 # Equals-sign-delimited flags, eg --foo=bar or -f=bar
-                if '=' in token:
-                    token, _, value = token.partition('=')
-                    debug("Splitting x=y expr {!r} into tokens {!r} and {!r}".format(orig, token, value)) # noqa
+                if "=" in token:
+                    token, _, value = token.partition("=")
+                    msg = "Splitting x=y expr {!r} into tokens {!r} and {!r}"
+                    debug(msg.format(orig, token, value))
                     mutations.append((index + 1, value))
                 # Contiguous boolean short flags, e.g. -qv
                 elif not is_long_flag(token) and len(token) > 2:
@@ -116,15 +128,18 @@ class Parser(object):
                     # core-args pass, handling what are going to be task args)
                     have_flag = (
                         token in machine.context.flags
-                        and machine.current_state != 'unknown'
+                        and machine.current_state != "unknown"
                     )
                     if have_flag and machine.context.flags[token].takes_value:
-                        debug("{!r} is a flag for current context & it takes a value, giving it {!r}".format(token, rest)) # noqa
+                        msg = "{!r} is a flag for current context & it takes a value, giving it {!r}"  # noqa
+                        debug(msg.format(token, rest))
                         mutations.append((index + 1, rest))
                     else:
-                        rest = ['-{}'.format(x) for x in rest]
-                        debug("Splitting multi-flag glob {!r} into {!r} and {!r}".format( # noqa
-                            orig, token, rest))
+                        rest = ["-{}".format(x) for x in rest]
+                        msg = (
+                            "Splitting multi-flag glob {!r} into {!r} and {!r}"
+                        )  # noqa
+                        debug(msg.format(orig, token, rest))
                         for item in reversed(rest):
                             mutations.append((index + 1, item))
             # Here, we've got some possible mutations queued up, and 'token'
@@ -154,33 +169,29 @@ class Parser(object):
             machine.handle(token)
         machine.finish()
         result = machine.result
-        result.remainder = ' '.join(remainder)
+        result.remainder = " ".join(remainder)
         return result
 
 
 class ParseMachine(StateMachine):
-    initial_state = 'context'
+    initial_state = "context"
 
-    state('context', enter=['complete_flag', 'complete_context'])
-    state('unknown', enter=['complete_flag', 'complete_context'])
-    state('end', enter=['complete_flag', 'complete_context'])
+    state("context", enter=["complete_flag", "complete_context"])
+    state("unknown", enter=["complete_flag", "complete_context"])
+    state("end", enter=["complete_flag", "complete_context"])
 
+    transition(from_=("context", "unknown"), event="finish", to="end")
     transition(
-        from_=('context', 'unknown'),
-        event='finish',
-        to='end',
+        from_="context",
+        event="see_context",
+        action="switch_to_context",
+        to="context",
     )
     transition(
-        from_='context',
-        event='see_context',
-        action='switch_to_context',
-        to='context',
-    )
-    transition(
-        from_=('context', 'unknown'),
-        event='see_unknown',
-        action='store_only',
-        to='unknown',
+        from_=("context", "unknown"),
+        event="see_unknown",
+        action="store_only",
+        to="unknown",
     )
 
     def changing_state(self, from_, to):
@@ -203,10 +214,7 @@ class ParseMachine(StateMachine):
     def waiting_for_flag_value(self):
         # Do we have a current flag, and does it expect a value (vs being a
         # bool/toggle)?
-        takes_value = (
-            self.flag and
-            self.flag.takes_value
-        )
+        takes_value = self.flag and self.flag.takes_value
         if not takes_value:
             return False
         # OK, this flag is one that takes values.
@@ -231,7 +239,7 @@ class ParseMachine(StateMachine):
         debug("Handling token: {!r}".format(token))
         # Handle unknown state at the top: we don't care about even
         # possibly-valid input if we've encountered unknown input.
-        if self.current_state == 'unknown':
+        if self.current_state == "unknown":
             debug("Top-of-handle() see_unknown({!r})".format(token))
             self.see_unknown(token)
             return
@@ -244,7 +252,11 @@ class ParseMachine(StateMachine):
             self.switch_to_flag(token, inverse=True)
         # Value for current flag
         elif self.waiting_for_flag_value:
-            debug("We're waiting for a flag value so {!r} must be it?".format(token)) # noqa
+            debug(
+                "We're waiting for a flag value so {!r} must be it?".format(
+                    token
+                )
+            )  # noqa
             self.see_value(token)
         # Positional args (must come above context-name check in case we still
         # need a posarg and the user legitimately wants to give it a value that
@@ -260,20 +272,18 @@ class ParseMachine(StateMachine):
         elif self.initial and token in self.initial.flags:
             debug("Saw (initial-context) flag {!r}".format(token))
             flag = self.initial.flags[token]
-            # TODO: handle ambiguity? Right now, flags in the context that
-            # shadow initial-context flags would always naturally "win" by
-            # being higher up in this if/elsif/etc chain. Ideally we'd complain
-            # to avoid users shooting themselves in the foot?
-            # Flags of this type that take a value are always given the current
-            # context's name as a string value.
-            # TODO: document this in the parser docs
-            if flag.takes_value:
+            # Special-case for core --help flag: context name is used as value.
+            if flag.name == "help":
                 flag.value = self.context.name
+                msg = "Saw --help in a per-task context, setting task name ({!r}) as its value"  # noqa
+                debug(msg.format(flag.value))
+            # All others: just enter the 'switch to flag' parser state
             else:
-                # TODO: handle inverse flags, other flag types?
-                flag.value = True
-            msg = "Setting (initial-context) flag {!r} to value {!r}"
-            debug(msg.format(flag, flag.value))
+                # TODO: handle inverse core flags too? There are none at the
+                # moment (e.g. --no-dedupe is actually 'no_dedupe', not a
+                # default-False 'dedupe') and it's up to us whether we actually
+                # put any in place.
+                self.switch_to_flag(token)
         # Unknown
         else:
             if not self.ignore_unknown:
@@ -289,13 +299,15 @@ class ParseMachine(StateMachine):
         self.result.unparsed.append(token)
 
     def complete_context(self):
-        debug("Wrapping up context {!r}".format(
-            self.context.name if self.context else self.context
-        ))
+        debug(
+            "Wrapping up context {!r}".format(
+                self.context.name if self.context else self.context
+            )
+        )
         # Ensure all of context's positional args have been given.
         if self.context and self.context.missing_positional_args:
             err = "'{}' did not receive required positional arguments: {}"
-            names = ', '.join(
+            names = ", ".join(
                 "'{}'".format(x.name)
                 for x in self.context.missing_positional_args
             )
@@ -308,9 +320,7 @@ class ParseMachine(StateMachine):
         debug("Moving to context {!r}".format(name))
         debug("Context args: {!r}".format(self.context.args))
         debug("Context flags: {!r}".format(self.context.flags))
-        debug("Context inverse_flags: {!r}".format(
-            self.context.inverse_flags
-        ))
+        debug("Context inverse_flags: {!r}".format(self.context.inverse_flags))
 
     def complete_flag(self):
         if self.flag:
@@ -371,7 +381,16 @@ class ParseMachine(StateMachine):
         # Set flag/arg obj
         flag = self.context.inverse_flags[flag] if inverse else flag
         # Update state
-        self.flag = self.context.flags[flag]
+        try:
+            self.flag = self.context.flags[flag]
+        except KeyError as e:
+            # Try fallback to initial/core flag
+            try:
+                self.flag = self.initial.flags[flag]
+            except KeyError:
+                # If it wasn't in either, raise the original context's
+                # exception, as that's more useful / correct.
+                raise e
         debug("Moving to flag {!r}".format(self.flag))
         # Bookkeeping for iterable-type flags (where the typical 'value
         # non-empty/nondefault -> clearly it got its value already' test is
@@ -412,6 +431,7 @@ class ParseResult(list):
 
     .. versionadded:: 1.0
     """
+
     def __init__(self, *args, **kwargs):
         super(ParseResult, self).__init__(*args, **kwargs)
         self.remainder = ""
